#pragma once
#include <easy2d/e2dbase.h>
#include <easy2d/e2dshape.h>
#include <easy2d/e2dtext.h>
#include <easy2d/e2dlistener.h>

namespace easy2d 
{

class Action;
class KeyFrame;
class Transition;
class SceneManager;

class Node :
	public Object
{
	friend class Scene;
	friend class Transition;
	friend class SceneManager;

public:
	// ڵ
	struct Property
	{
		bool visable;		// ɼ
		Point pos;			// 
		Size size;			// 
		float opacity;		// ͸
		Point anchor;		// ê
		Vector2 scale;		// 
		float rotation;		// תǶ
		Vector2 skewAngle;	// бǶ
	};

public:
	Node();

	virtual ~Node();

	// ½ڵ
	virtual void onUpdate() {}

	// Ⱦڵ
	virtual void onRender() {}

	// ȡڵʾ״̬
	bool isVisible() const;

	// ȡڵ
	String getName() const;

	// ȡڵƵ Hash ֵ
	size_t getHashName() const;

	// ȡڵͼ˳
	int getOrder() const;

	// ȡڵ
	float getPosX() const;

	// ȡڵ
	float getPosY() const;

	// ȡڵ
	Point getPos() const;

	// ȡڵ
	float getWidth() const;

	// ȡڵ߶
	float getHeight() const;

	// ȡڵȣţ
	float getRealWidth() const;

	// ȡڵ߶ȣţ
	float getRealHeight() const;

	// ȡڵСţ
	Size getRealSize() const;

	// ȡڵê
	float getAnchorX() const;

	// ȡڵê
	float getAnchorY() const;

	// ȡڵС
	Size getSize() const;

	// ȡڵű
	float getScaleX() const;

	// ȡڵű
	float getScaleY() const;

	// ȡڵбǶ
	float getSkewX() const;

	// ȡڵбǶ
	float getSkewY() const;

	// ȡڵתǶ
	float getRotation() const;

	// ȡڵ͸
	float getOpacity() const;

	// ȡڵ
	Property getProperty() const;

	// ȡ߿
	virtual Rect getBounds() const;

	// ȡаΧ
	virtual Rect getBoundingBox() const;

	// ȡά任
	Matrix32 getTransform() const;

	// ȡά任
	Matrix32 getInverseTransform() const;

	// ȡڵ
	Node * getParent() const;

	// ȡڵڳ
	Scene * getParentScene() const;

	// Ƿ
	virtual bool containsPoint(Point const& point) const;

	// ýڵǷʾ
	void setVisible(
		bool value
	);

	//  onUpdate 
	void setAutoUpdate(
		bool bAutoUpdate
	);

	// ýڵ
	void setName(
		const String& name
	);

	// ýڵ
	void setPosX(
		float x
	);

	// ýڵ
	void setPosY(
		float y
	);

	// ýڵ
	void setPos(
		const Point & point
	);

	// ýڵ
	void setPos(
		float x,
		float y
	);

	// ڵ̶
	void setPosFixed(
		bool fixed
	);

	// ƶڵ
	void movePosX(
		float x
	);

	// ƶڵ
	void movePosY(
		float y
	);

	// ƶڵ
	void movePos(
		float x,
		float y
	);

	// ƶڵ
	void movePos(
		const Vector2 & v
	);

	// ýڵͼ˳
	// ĬΪ 0
	void setOrder(
		int order
	);

	// úű
	// ĬΪ 1.0f
	void setScaleX(
		float scaleX
	);

	// ű
	// ĬΪ 1.0f
	void setScaleY(
		float scaleY
	);

	// ű
	// ĬΪ (1.0f, 1.0f)
	void setScale(
		float scaleX,
		float scaleY
	);

	// ű
	// ĬΪ 1.0f
	void setScale(
		float scale
	);

	// úбǶ
	// ĬΪ 0
	void setSkewX(
		float angleX
	);

	// бǶ
	// ĬΪ 0
	void setSkewY(
		float angleY
	);

	// бǶ
	// ĬΪ (0, 0)
	void setSkew(
		float angleX,
		float angleY
	);

	// תǶ
	// ĬΪ 0
	void setRotation(
		float rotation
	);

	// ͸
	// ĬΪ 1.0f, Χ [0, 1]
	void setOpacity(
		float opacity
	);

	// êĺλ
	// ĬΪ 0, Χ [0, 1]
	void setAnchorX(
		float anchorX
	);

	// êλ
	// ĬΪ 0, Χ [0, 1]
	void setAnchorY(
		float anchorY
	);

	// êλ
	// ĬΪ (0, 0), Χ [0, 1]
	void setAnchor(
		float anchorX,
		float anchorY
	);

	// ޸Ľڵ
	void setWidth(
		float width
	);

	// ޸Ľڵ߶
	void setHeight(
		float height
	);

	// ޸ĽڵС
	void setSize(
		float width,
		float height
	);

	// ޸ĽڵС
	void setSize(
		Size size
	);

	// ýڵ
	void setProperty(
		Property prop
	);

	// ӽڵ
	void addChild(
		Node * child,
		int order = 0	/* Ⱦ˳ */
	);

	// Ӷӽڵ
	void addChild(
		const std::vector<Node*>& nodes,	/* ڵ */
		int order = 0						/* Ⱦ˳ */
	);

	// ȡͬӽڵ
	std::vector<Node*> getChildren(
		const String& name
	) const;

	// ȡͬӽڵ
	Node* getChild(
		const String& name
	) const;

	// ȡӽڵ
	const std::vector<Node*>& getAllChildren() const;

	// ȡӽڵ
	int getChildrenCount() const;

	// Ƴӽڵ
	bool removeChild(
		Node* child
	);

	// Ƴͬӽڵ
	void removeChildren(
		const String& childName
	);

	// ӸڵƳ
	void removeFromParent();

	// Ƴнڵ
	void removeAllChildren();

	// ַ¼
	virtual void dispatch(Event* evt);

	// ִж
	void runAction(
		Action * action
	);

	// 
	void resumeAction(
		const String& name
	);

	// ͣ
	void pauseAction(
		const String& name
	);

	// ֹͣ
	void stopAction(
		const String& name
	);

	// ͣ
	void resumeAllActions();

	// ͣж
	void pauseAllActions();

	// ֹͣж
	void stopAllActions();

	// 
	Listener* addListener(
		const Listener::Callback& func,	/* ûʱִк */
		const String& name = L"",		/*  */
		bool paused = false				/* Ƿͣ */
	);

	// ײ
	void addListener(
		ListenerBase* listener		/*  */
	);

	// Ƴ
	void removeListener(
		ListenerBase* listener		/*  */
	);

	// 
	void startListener(
		const String& name
	);

	// ֹͣ
	void stopListener(
		const String& name
	);

	// Ƴ
	void removeListener(
		const String& name
	);

	// м
	void startAllListeners();

	// ֹͣм
	void stopAllListeners();

	// Ƴм
	void removeAllListeners();

	// ޸ĽڵĬêλ
	static void setDefaultAnchor(
		float defaultAnchorX,
		float defaultAnchorY
	);

protected:
	// ½ڵ
	void _update();

	// Ⱦڵ
	void _render();

	// ýڵڳ
	void _setParentScene(
		Scene * scene
	);

	// ¶ά任
	void _updateTransform() const;

	// ¶ά任
	void _updateInverseTransform() const;

	// ӽڵ
	void _sortChildren();

	// ½ڵ͸
	void _updateOpacity();

	// ¼
	void __updateListeners(Event* evt);

	// ռ
	void __clearListeners();

protected:
	bool		_visible;
	bool		_autoUpdate;
	bool		_needSort;
	bool		_positionFixed;
	int			_order;
	float		_rotation;
	float		_displayOpacity;
	float		_realOpacity;
	Point		_pos;
	Size		_size;
	Vector2		_scale;
	Vector2		_skewAngle;
	Point		_anchor;
	String*		_name;
	size_t		_hashName;
	Scene*		_parentScene;
	Node*		_parent;
	
	std::vector<Node*>	_children;
	std::vector<ListenerBase*> _listeners;

	mutable bool		_dirtyTransform;
	mutable Matrix32	_transform;
	mutable bool		_dirtyInverseTransform;
	mutable Matrix32	_inverseTransform;
};


// 
class Scene :
	public Node
{
public:
	Scene();

	virtual ~Scene();

	// дڽʱԶִ
	virtual void onEnter() {}

	// д뿪ʱԶִ
	virtual void onExit() {}

	// дڹرմʱִУ false ֹڹرգ
	virtual bool onCloseWindow() { return true; }
};


// 
class Sprite :
	public Node
{
public:
	Sprite();

	explicit Sprite(
		Image * image
	);

	explicit Sprite(
		const String& filePath	/* ͼƬļ· */
	);

	explicit Sprite(
		const Resource& res		/* ͼƬԴ */
	);

	explicit Sprite(
		KeyFrame* frame			/* ؼ֡ */
	);

	Sprite(
		int resNameId,			/* ͼƬԴ */
		const String& resType	/* ͼƬԴ */
	);

	Sprite(
		const String& filePath,	/* ͼƬļ· */
		const Rect& cropRect	/* ü */
	);

	Sprite(
		const Resource& res,	/* ͼƬԴ */
		const Rect& cropRect	/* ü */
	);

	Sprite(
		int resNameId,			/* ͼƬԴ */
		const String& resType,	/* ͼƬԴ */
		const Rect& cropRect	/* ü */
	);

	virtual ~Sprite();

	// ͼƬļ
	bool open(
		const String& filePath
	);

	// ͼƬԴ
	bool open(
		const Resource& res		/* ͼƬԴ */
	);

	// ͼƬԴ
	bool open(
		int resNameId,			/* ͼƬԴ */
		const String& resType	/* ͼƬԴ */
	);

	// ȡü
	Rect getCropRect() const;

	// ͼƬüΪ
	void crop(
		const Rect& cropRect	/* üΣվ򲻲ü */
	);

	// ȡͼƬ
	Image* getImage() const;

	// ͼƬ
	void setImage(
		Image* image,
		bool resetCropRect = true	/* òü */
	);

	// ùؼ֡
	void setKeyFrame(
		KeyFrame* frame
	);

	// ȡزֵʽ
	InterpolationMode getInterpolationMode() const;

	// زֵʽ
	void setInterpolationMode(InterpolationMode mode);

	// Ⱦ
	virtual void onRender() override;

protected:
	Image*	_image;
	Rect	_cropRect;
	InterpolationMode _interpolationMode;
};


// ıڵ
class Text
	: public Node
{
public:
	Text();

	explicit Text(
		const String& text,									/*  */
		const TextStyle& textStyle = TextStyle(),			/* ıʽ */
		const DrawingStyle& drawingStyle = DrawingStyle()	/* ͼʽ */
	);

	explicit Text(
		TextLayout* layout,									/* ֲ */
		const DrawingStyle& drawingStyle = DrawingStyle()	/* ͼʽ */
	);

	virtual ~Text();

	// ȡı
	TextLayout* getLayout() const;

	// ȡı
	String getText() const;

	// ȡ
	Font getFont() const;

	// ȡıʽ
	TextStyle getTextStyle() const;

	// ȡȾʽ
	DrawingStyle getDrawingStyle() const;

	// ȡıʾ
	int getLineCount() const;

	// ı
	void setTextLayout(
		TextLayout* layout
	);

	// ı
	void setText(
		const String& text
	);

	// ıʽ
	void setTextStyle(
		const TextStyle& style
	);

	// Ⱦʽ
	void setDrawingStyle(
		DrawingStyle style
	);

	// 
	void setFont(
		const Font& font
	);

	// 
	void setFontFamily(
		const String& family
	);

	// ֺ
	void setFontSize(
		float size
	);

	// ϸֵ
	void setFontWeight(
		UINT weight
	);

	// б
	void setItalic(
		bool italic
	);

	// 򿪻رıԶ
	void setWrapping(
		bool wrapping
	);

	// ıԶеĿ
	void setWrappingWidth(
		float wrappingWidth
	);

	// мࣨĬΪ 0
	void setLineSpacing(
		float lineSpacing
	);

	// ö뷽ʽ
	void setAlignment(
		TextAlign align
	);

	// »
	void setUnderline(
		bool hasUnderline
	);

	// ɾ
	void setStrikethrough(
		bool hasStrikethrough
	);

	// ɫ
	void setFillColor(
		Color color
	);

	// ɫ
	void setStrokeColor(
		Color strokeColor
	);

	// ߿
	void setStrokeWidth(
		float strokeWidth
	);

	// ཻʽ
	void setLineJoin(
		LineJoin lineJoin
	);

	// Ⱦ
	virtual void onRender() override;

protected:
	void updateLayout();

protected:
	TextLayout*	_layout;
	DrawingStyle _style;
};


// ť
// ѷʹ ButtonListener 
class Button :
	public Node
{
public:
	using Callback = Function<void()>;

	Button();

	explicit Button(
		Node * normal,					/* ͨ״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit Button(
		Node * normal,					/* ͨ״̬ */
		Node * selected,				/* 갴״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit Button(
		Node * normal,					/* ͨ״̬ */
		Node * mouseover,				/* ״̬ */
		Node * selected,				/* 갴״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit Button(
		Node * normal,					/* ͨ״̬ */
		Node * mouseover,				/* ״̬ */
		Node * selected,				/* ״̬ */
		Node * disabled,				/* ť״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	// ȡť״̬ûǽ
	bool isEnable() const;

	// ðťû
	void setEnable(
		bool enable
	);

	// һʾİť
	virtual void setNormal(
		Node * normal
	);

	// 밴ťʱʾİť
	virtual void setMouseOver(
		Node * mouseover
	);

	// 갴°ťʱʾİť
	virtual void setSelected(
		Node * selected
	);

	// ðťʱʾİť
	virtual void setDisabled(
		Node * disabled
	);

	// ðťִк
	void setClickFunc(
		const Callback& func
	);

protected:
	// ť״̬ö
	enum class ButtonState { Normal, Mouseover, Selected };

	// ðť״̬
	virtual void _setState(ButtonState state);

	// ˢ°ťʾ
	virtual void _updateVisiable();

	// ִаť
	virtual void _runCallback();

	void handle(ButtonEvent evt);

protected:
	bool		_enable;
	Node*		_normal;
	Node*		_mouseover;
	Node*		_selected;
	Node*		_disabled;
	ButtonState	_state;
	Callback	_func;
};


// ذť
// ѷʹ ToggleButtonListener 
class ToggleButton :
	public Button
{
public:
	ToggleButton();

	explicit ToggleButton(
		Node * onNormal,				/* ťʱͨ״̬ */
		Node * offNormal,				/* ťرʱͨ״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit ToggleButton(
		Node * onNormal,				/* ťʱͨ״̬ */
		Node * offNormal,				/* ťرʱͨ״̬ */
		Node * onSelected,				/* ťʱ갴״̬ */
		Node * offSelected,				/* ťرʱ갴״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit ToggleButton(
		Node * onNormal,				/* ťʱͨ״̬ */
		Node * offNormal,				/* ťرʱͨ״̬ */
		Node * onMouseOver,				/* ťʱ״̬ */
		Node * offMouseOver,			/* ťرʱ״̬ */
		Node * onSelected,				/* ťʱ갴״̬ */
		Node * offSelected,				/* ťرʱ갴״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	explicit ToggleButton(
		Node * onNormal,				/* ťʱͨ״̬ */
		Node * offNormal,				/* ťرʱͨ״̬ */
		Node * onMouseOver,				/* ťʱ״̬ */
		Node * offMouseOver,			/* ťرʱ״̬ */
		Node * onSelected,				/* ťʱ갴״̬ */
		Node * offSelected,				/* ťرʱ갴״̬ */
		Node * onDisabled,				/* ťʱ״̬ */
		Node * offDisabled,				/* ťرʱ״̬ */
		const Callback& func = nullptr	/* ťִк */
	);

	// ȡ״̬򿪻رգ
	bool getState() const;

	// ÿذť״̬򿪻رգ
	void setState(
		bool bState
	);

	// ðť״̬ʾİť
	virtual void setNormal(
		Node* normal
	) override;

	// ðť״̬£밴ťʱʾİť
	virtual void setMouseOver(
		Node* mouseover
	) override;

	// ðť״̬£갴°ťʱʾİť
	virtual void setSelected(
		Node* selected
	) override;

	// ðť״̬£ʱʾİť
	virtual void setDisabled(
		Node* disabled
	) override;

	// ðťر״̬ʾİť
	void setNormalOff(
		Node * normal
	);

	// ðťر״̬£밴ťʱʾİť
	void setMouseOverOff(
		Node * mouseover
	);

	// ðťر״̬£갴°ťʱʾİť
	void setSelectedOff(
		Node * selected
	);

	// ðťر״̬£ťʱʾİť
	void setDisabledOff(
		Node * disabled
	);

protected:
	// ˢ°ť
	void _updateState();

	// ִаť
	void _runCallback() override;

protected:
	bool _toggle;
	Node* _normalOn;
	Node* _mouseoverOn;
	Node* _selectedOn;
	Node* _disabledOn;
	Node* _normalOff;
	Node* _mouseoverOff;
	Node* _selectedOff;
	Node* _disabledOff;
};


// ˵
class Menu :
	public Node
{
public:
	Menu();

	explicit Menu(
		const std::vector<Button*>& buttons	/* ť */
	);

	// ȡ˵Ƿ
	bool isEnable() const;

	// ȡ˵еİť
	size_t getButtonCount() const;

	// ò˵û
	void setEnable(
		bool enable
	);

	// Ӱť
	void addButton(
		Button * button
	);

	// Ƴť
	bool removeButton(
		Button * button
	);

	// ȡаť
	const std::vector<Button*>& getAllButtons() const;

protected:
	bool _enable;
	std::vector<Button*> _buttons;
};


// ״ڵ
class ShapeNode :
	public Node
{
public:
	// ֱ߽ڵ
	static ShapeNode* createLine(
		Point begin,
		Point end,
		DrawingStyle style = DrawingStyle{}
	);

	// νڵ
	static ShapeNode* createRect(
		const Size& size,		// ο
		DrawingStyle style = DrawingStyle{}
	);

	// ԲǾνڵ
	static ShapeNode* createRoundedRect(
		const Size& size,		// ο
		const Vector2& radius,	// Բǰ뾶
		DrawingStyle style = DrawingStyle{}
	);

	// Բνڵ
	static ShapeNode* createCircle(
		float radius,			// 뾶
		DrawingStyle style = DrawingStyle{}
	);

	// Բνڵ
	static ShapeNode* createEllipse(
		const Vector2& radius,	// 뾶
		DrawingStyle style = DrawingStyle{}
	);

	// νڵ
	static ShapeNode* createPolygon(
		std::initializer_list<Point> vertices,	// ζ
		DrawingStyle style = DrawingStyle{}
	);

	// νڵ
	static ShapeNode* createPolygon(
		const Point* vertices,
		int count,
		DrawingStyle style = DrawingStyle{}
	);

	ShapeNode(
		Shape* shape = nullptr,
		DrawingStyle style = DrawingStyle{}
	);

	virtual ~ShapeNode();

	// ȡ״
	Shape* getShape() const;

	// ״
	void setShape(Shape* shape);

	// ȡɫ
	Color getFillColor() const;

	// ɫ
	void setFillColor(
		Color fillColor
	);

	// ȡɫ
	Color getStrokeColor() const;

	// ɫ
	void setStrokeColor(
		Color strokeColor
	);

	// ȡ
	float getStrokeWidth() const;

	// 
	void setStrokeWidth(
		float strokeWidth
	);

	// ཻʽ
	void setLineJoin(
		LineJoin lineJoin
	);

	// ȡͼģʽ
	DrawingStyle::Mode getDrawingMode() const;

	// ûͼģʽ
	void setDrawingMode(DrawingStyle::Mode mode);

	// ȡͼʽ
	DrawingStyle getDrawingStyle() const;

	// ûͼʽ
	void setDrawingStyle(DrawingStyle style);

	virtual Rect getBounds() const override;

	virtual Rect getBoundingBox() const override;

	virtual bool containsPoint(Point const& point) const override;

	virtual void onRender() override;

protected:
	Shape*			_shape;
	Rect			_bounds;
	DrawingStyle	_style;
};


class CanvasBase;

// 
class CanvasBrush
	: public Object
{
	friend CanvasBase;

public:
	// ״
	void drawShape(
		Shape* shape
	);

	// ͼƬ
	void drawImage(
		Image* image,					/* ͼ */
		const Point& pos = Point(),		/* λ */
		const Rect& cropRect = Rect{}	/* ü */
	);

	// ͼƬ
	void drawImage(
		Image* image,					/* ͼ */
		const Rect& destRect,			/*  */
		const Rect& cropRect = Rect{}	/* ü */
	);

	// ƹؼ֡
	void drawImage(
		KeyFrame* frame,				/* ؼ֡ */
		const Point& pos = Point()		/* λ */
	);

	// ƹؼ֡
	void drawImage(
		KeyFrame* frame,				/* ؼ֡ */
		const Rect& destRect			/*  */
	);

	// ı
	void drawText(
		TextLayout* layout,
		const Point& pos = Point()
	);

	// ı
	void drawText(
		const String& text,
		const Point& pos = Point(),
		const TextStyle& style = TextStyle()
	);

	// ȡɫ
	Color getFillColor() const;

	// ɫ
	void setFillColor(
		Color fillColor
	);

	// ȡɫ
	Color getStrokeColor() const;

	// ɫ
	void setStrokeColor(
		Color strokeColor
	);

	// ȡ
	float getStrokeWidth() const;

	// 
	void setStrokeWidth(
		float strokeWidth
	);

	// ཻʽ
	void setLineJoin(
		LineJoin lineJoin
	);

	// ȡͼģʽ
	DrawingStyle::Mode getDrawingMode() const;

	// ûͼģʽ
	void setDrawingMode(DrawingStyle::Mode mode);

	// ȡͼʽ
	DrawingStyle getDrawingStyle() const;

	// ûͼʽ
	void setDrawingStyle(DrawingStyle style);

	// ȡ͸
	float getOpacity() const;

	// ͸ȣΧ0 ~ 1
	void setOpacity(float opacity);

	// ȡ
	Point getPos() const;

	// 
	void setPos(
		const Point& point
	);

	// ƶ
	void movePos(
		const Vector2& point
	);

	// ȡתǶ
	float getRotation() const;

	// תǶ
	void setRotation(
		float rotation
	);

	// ȡű
	Vector2 getScale() const;

	// ű
	void setScale(
		const Vector2& scale
	);

	// ȡڵбǶ
	Vector2 getSkew() const;

	// бǶ
	void setSkew(
		const Vector2& skew
	);

	// ȡʱ任
	Matrix32 getTransform() const;

	// ȡͼʱزֵʽ
	InterpolationMode getInterpolationMode() const;

	// ûͼʱزֵʽ
	void setInterpolationMode(InterpolationMode mode);

	// ջ
	void clear();

	// ָɫջ
	void clear(const Color& color);

	virtual ~CanvasBrush();

protected:
	CanvasBrush(
		ID2D1RenderTarget* rt,
		ID2D1SolidColorBrush* brush
	);

	void _updateTransform();

protected:
	bool _dirtyTransform;
	ID2D1RenderTarget* _rt;
	ID2D1SolidColorBrush* _brush;
	ID2D1DrawingStateBlock* _state;
	InterpolationMode _interpolationMode;
	float _opacity;
	Point _pos;
	float _rotation;
	Vector2 _scale;
	Vector2 _skew;
	DrawingStyle _style;
};


// 
class CanvasBase
	: public Node
{
public:
	CanvasBase(const Size& size);

	virtual ~CanvasBase();

	// »ϴ
	void redraw();

	// 軭Сջᵼ»ˢʧЧ
	void resizeAndClear(Size size);

	// ȡزֵʽ
	InterpolationMode getInterpolationMode() const;

	// زֵʽ
	void setInterpolationMode(InterpolationMode mode);

	// ȡͼ
	Image* getImage() const;

	virtual void onRender() override;

protected:
	// ͼ
	virtual void draw(CanvasBrush* brush) = 0;

private:
	void _initialize();

	void _discardResources();

private:
	Image* _image;
	ID2D1RenderTarget* _rt;
	ID2D1SolidColorBrush* _brush;
	InterpolationMode _interpolationMode;
};

// 
class Canvas
	: public CanvasBase
{
public:
	Canvas(const Size& size);

	// ͼ
	void draw(const Function<void(CanvasBrush*)>& drawing);

protected:
	virtual void draw(CanvasBrush* brush) override;

private:
	Function<void(CanvasBrush*)> _drawing;
};

}